@@ -40,7 +40,7 @@ class WebsocketClient(Thread):
         suppress_origin: bool = False,
         ping_interval: Union[int, float] = 0,
         ping_timeout: Optional[Union[int, float]] = None,
-        ping_payload: str = ""
+        ping_payload: str = "",
     ):
         if rootlogger.level <= TRACE:
             enableTrace(True, log)
@@ -81,13 +81,13 @@ class WebsocketClient(Thread):
             ping_interval=ping_interval,
             ping_timeout=ping_timeout,
             ping_payload=ping_payload,
-            **proxy_options
+            **proxy_options,
         )
 
         self._id += 1
         super().__init__(
             name=f"Thread-{self.__class__.__name__}-{self._id}",
-            daemon=True
+            daemon=True,
         )
 
     def _ws_init(self, url, subprotocols, header, cookie):
@@ -103,7 +103,7 @@ class WebsocketClient(Thread):
             on_pong=self.on_pong,
             on_message=self.on_message,
             on_cont_message=self.on_cont_message,
-            on_data=self.on_data
+            on_data=self.on_data,
         )
 
     def run(self) -> None:
@@ -124,7 +124,7 @@ class WebsocketClient(Thread):
         subprotocols: Optional[List[str]] = None,
         header: Optional[Union[List, Dict]] = None,
         cookie: Optional[str] = None,
-        closeopts: Optional[Dict] = None
+        closeopts: Optional[Dict] = None,
     ) -> None:
         with self._reconnect_lock:
             # ws connection is not active (anymore)
@@ -137,11 +137,11 @@ class WebsocketClient(Thread):
                 url=self.ws.url if url is None else url,
                 subprotocols=self.ws.subprotocols if subprotocols is None else subprotocols,
                 header=self.ws.header if header is None else header,
-                cookie=self.ws.cookie if cookie is None else cookie
+                cookie=self.ws.cookie if cookie is None else cookie,
             )
 
     def close(self, status: int = STATUS_NORMAL, reason: Union[str, bytes] = "", timeout: int = 3) -> None:
-        if type(reason) is str:
+        if isinstance(reason, str):
             reason = bytes(reason, encoding="utf-8")
         self.ws.close(status=status, reason=reason, timeout=timeout)
         if self.is_alive() and current_thread() is not self:
