@@ -1,80 +1,77 @@
 import copy
-import datetime
 import itertools
 import logging
 from collections import defaultdict
-from contextlib import contextmanager
-from pathlib import Path
+from contextlib import contextmanager, suppress
+from datetime import datetime
 from time import time
-from typing import Dict, Optional
+from typing import Any, Dict, List, Optional, Tuple
 from urllib.parse import urlparse, urlunparse
 
-from streamlink import PluginError, StreamError
+from requests import Response
+
+from streamlink.exceptions import PluginError, StreamError
+from streamlink.session import Streamlink
 from streamlink.stream.dash_manifest import MPD, Representation, Segment, freeze_timeline
 from streamlink.stream.ffmpegmux import FFMPEGMuxer
 from streamlink.stream.segmented import SegmentedStreamReader, SegmentedStreamWorker, SegmentedStreamWriter
 from streamlink.stream.stream import Stream
 from streamlink.utils.l10n import Language
 from streamlink.utils.parse import parse_xml
+from streamlink.utils.times import now
 
 
 log = logging.getLogger(__name__)
 
-UTC = datetime.timezone.utc
 
-
-class DASHStreamWriter(SegmentedStreamWriter):
+class DASHStreamWriter(SegmentedStreamWriter[Segment, Response]):
     reader: "DASHStreamReader"
     stream: "DASHStream"
 
-    @staticmethod
-    def _get_segment_name(segment: Segment) -> str:
-        return Path(urlparse(segment.url).path).resolve().name
-
-    def fetch(self, segment: Segment, retries: Optional[int] = None):
-        if self.closed or not retries:
+    def fetch(self, segment: Segment):
+        if self.closed:
             return
 
+        name = segment.name
+        available_in = segment.available_in
+        if available_in > 0:
+            log.debug(f"{self.reader.mime_type} segment {name}: waiting {available_in:.01f}s ({segment.availability})")
+            if not self.wait(available_in):
+                log.debug(f"{self.reader.mime_type} segment {name}: cancelled")
+                return
+        log.debug(f"{self.reader.mime_type} segment {name}: downloading ({segment.availability})")
+
+        request_args = copy.deepcopy(self.reader.stream.args)
+        headers = request_args.pop("headers", {})
+
+        if segment.byterange:
+            start, length = segment.byterange
+            end = str(start + length - 1) if length else ""
+            headers["Range"] = f"bytes={start}-{end}"
+
         try:
-            request_args = copy.deepcopy(self.reader.stream.args)
-            headers = request_args.pop("headers", {})
-            now = datetime.datetime.now(tz=UTC)
-            if segment.available_at > now:
-                time_to_wait = (segment.available_at - now).total_seconds()
-                fname = self._get_segment_name(segment)
-                log.debug(f"Waiting for segment: {fname} ({time_to_wait:.01f}s)")
-                if not self.wait(time_to_wait):
-                    log.debug(f"Waiting for segment: {fname} aborted")
-                    return
-
-            if segment.byterange:
-                start, length = segment.byterange
-                end = str(start + length - 1) if length else ""
-                headers["Range"] = f"bytes={start}-{end}"
-
             return self.session.http.get(
                 segment.url,
                 timeout=self.timeout,
                 exception=StreamError,
                 headers=headers,
+                retries=self.retries,
                 **request_args,
             )
         except StreamError as err:
-            log.error(f"Failed to open segment {segment.url}: {err}")
-            return self.fetch(segment, retries - 1)
+            log.error(f"{self.reader.mime_type} segment {name}: failed ({err})")
 
     def write(self, segment, res, chunk_size=8192):
-        name = self._get_segment_name(segment)
         for chunk in res.iter_content(chunk_size):
             if self.closed:
-                log.warning(f"Download of segment: {name} aborted")
+                log.warning(f"{self.reader.mime_type} segment {segment.name}: aborted")
                 return
             self.reader.buffer.write(chunk)
 
-        log.debug(f"Download of segment: {name} complete")
+        log.debug(f"{self.reader.mime_type} segment {segment.name}: completed")
 
 
-class DASHStreamWorker(SegmentedStreamWorker):
+class DASHStreamWorker(SegmentedStreamWorker[Segment, Response]):
     reader: "DASHStreamReader"
     writer: "DASHStreamWriter"
     stream: "DASHStream"
@@ -82,7 +79,8 @@ class DASHStreamWorker(SegmentedStreamWorker):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.mpd = self.stream.mpd
-        self.period = self.stream.period
+
+        self.manifest_reload_retries = self.session.options.get("dash-manifest-reload-attempts")
 
     @contextmanager
     def sleeper(self, duration):
@@ -95,33 +93,31 @@ class DASHStreamWorker(SegmentedStreamWorker):
         if time_to_sleep > 0:
             self.wait(time_to_sleep)
 
-    @staticmethod
-    def get_representation(mpd, representation_id, mime_type):
-        for aset in mpd.periods[0].adaptationSets:
-            for rep in aset.representations:
-                if rep.id == representation_id and rep.mimeType == mime_type:
-                    return rep
-
     def iter_segments(self):
         init = True
         back_off_factor = 1
         while not self.closed:
             # find the representation by ID
-            representation = self.get_representation(self.mpd, self.reader.representation_id, self.reader.mime_type)
+            representation = self.mpd.get_representation(self.reader.ident)
 
             if self.mpd.type == "static":
                 refresh_wait = 5
             else:
                 refresh_wait = max(
                     self.mpd.minimumUpdatePeriod.total_seconds(),
-                    self.mpd.periods[0].duration.total_seconds(),
+                    representation.period.duration.total_seconds() if representation else 0,
                 ) or 5
 
             with self.sleeper(refresh_wait * back_off_factor):
                 if not representation:
                     continue
 
-                for segment in representation.segments(init=init):
+                iter_segments = representation.segments(
+                    init=init,
+                    # sync initial timeline generation between audio and video threads
+                    timestamp=self.reader.timestamp if init else None,
+                )
+                for segment in iter_segments:
                     if self.closed:
                         break
                     yield segment
@@ -143,15 +139,22 @@ class DASHStreamWorker(SegmentedStreamWorker):
             return
 
         self.reader.buffer.wait_free()
-        log.debug("Reloading manifest ({0}:{1})".format(self.reader.representation_id, self.reader.mime_type))
-        res = self.session.http.get(self.mpd.url, exception=StreamError, **self.stream.args)
+        log.debug(f"Reloading manifest {self.reader.ident!r}")
+        res = self.session.http.get(
+            self.mpd.url,
+            exception=StreamError,
+            retries=self.manifest_reload_retries,
+            **self.stream.args,
+        )
 
-        new_mpd = MPD(self.session.http.xml(res, ignore_ns=True),
-                      base_url=self.mpd.base_url,
-                      url=self.mpd.url,
-                      timelines=self.mpd.timelines)
+        new_mpd = MPD(
+            self.session.http.xml(res, ignore_ns=True),
+            base_url=self.mpd.base_url,
+            url=self.mpd.url,
+            timelines=self.mpd.timelines,
+        )
 
-        new_rep = self.get_representation(new_mpd, self.reader.representation_id, self.reader.mime_type)
+        new_rep = new_mpd.get_representation(self.reader.ident)
         with freeze_timeline(new_mpd):
             changed = len(list(itertools.islice(new_rep.segments(), 1))) > 0
 
@@ -161,7 +164,7 @@ class DASHStreamWorker(SegmentedStreamWorker):
         return changed
 
 
-class DASHStreamReader(SegmentedStreamReader):
+class DASHStreamReader(SegmentedStreamReader[Segment, Response]):
     __worker__ = DASHStreamWorker
     __writer__ = DASHStreamWriter
 
@@ -169,11 +172,16 @@ class DASHStreamReader(SegmentedStreamReader):
     writer: "DASHStreamWriter"
     stream: "DASHStream"
 
-    def __init__(self, stream: "DASHStream", representation_id, mime_type, *args, **kwargs):
-        super().__init__(stream, *args, **kwargs)
-        self.mime_type = mime_type
-        self.representation_id = representation_id
-        log.debug("Opening DASH reader for: {0} ({1})".format(self.representation_id, self.mime_type))
+    def __init__(
+        self,
+        stream: "DASHStream",
+        representation: Representation,
+        timestamp: datetime,
+    ):
+        super().__init__(stream)
+        self.ident = representation.ident
+        self.mime_type = representation.mimeType
+        self.timestamp = timestamp
 
 
 class DASHStream(Stream):
@@ -185,30 +193,27 @@ class DASHStream(Stream):
 
     def __init__(
         self,
-        session,
+        session: Streamlink,
         mpd: MPD,
         video_representation: Optional[Representation] = None,
         audio_representation: Optional[Representation] = None,
-        period: float = 0,
-        **args,
+        **kwargs,
     ):
         """
-        :param streamlink.Streamlink session: Streamlink session instance
+        :param session: Streamlink session instance
         :param mpd: Parsed MPD manifest
         :param video_representation: Video representation
         :param audio_representation: Audio representation
-        :param period: Update period
-        :param args: Additional keyword arguments passed to :meth:`requests.Session.request`
+        :param kwargs: Additional keyword arguments passed to :meth:`requests.Session.request`
         """
 
         super().__init__(session)
         self.mpd = mpd
         self.video_representation = video_representation
         self.audio_representation = audio_representation
-        self.period = period
-        self.args = args
+        self.args = session.http.valid_request_args(**kwargs)
 
-    def __json__(self):
+    def __json__(self):  # noqa: PLW3201
         json = dict(type=self.shortname())
 
         if self.mpd.url:
@@ -230,51 +235,77 @@ class DASHStream(Stream):
         # the MPD URL has already been prepared by the initial request in `parse_manifest`
         return self.mpd.url
 
+    @staticmethod
+    def fetch_manifest(session: Streamlink, url_or_manifest: str, **request_args) -> Tuple[str, Dict[str, Any]]:
+        if url_or_manifest.startswith("<?xml"):
+            return url_or_manifest, {}
+
+        retries = session.options.get("dash-manifest-reload-attempts")
+        args = session.http.valid_request_args(**request_args)
+        res = session.http.get(url_or_manifest, retries=retries, **args)
+        manifest: str = res.text
+        url: str = res.url
+
+        urlp = list(urlparse(url))
+        urlp[2], _ = urlp[2].rsplit("/", 1)
+        base_url: str = urlunparse(urlp)
+
+        return manifest, dict(url=url, base_url=base_url)
+
+    @staticmethod
+    def parse_mpd(manifest: str, mpd_params: Dict[str, Any]) -> MPD:
+        node = parse_xml(manifest, ignore_ns=True)
+
+        return MPD(node, **mpd_params)
+
     @classmethod
     def parse_manifest(
         cls,
-        session,
+        session: Streamlink,
         url_or_manifest: str,
-        **args,
+        period: int = 0,
+        with_video_only: bool = False,
+        with_audio_only: bool = False,
+        **kwargs,
     ) -> Dict[str, "DASHStream"]:
         """
         Parse a DASH manifest file and return its streams.
 
-        :param streamlink.Streamlink session: Streamlink session instance
+        :param session: Streamlink session instance
         :param url_or_manifest: URL of the manifest file or an XML manifest string
-        :param args: Additional keyword arguments passed to :meth:`requests.Session.request`
+        :param period: Which MPD period to use (index number) for finding representations
+        :param with_video_only: Also return video-only streams, otherwise only return muxed streams
+        :param with_audio_only: Also return audio-only streams, otherwise only return muxed streams
+        :param kwargs: Additional keyword arguments passed to :meth:`requests.Session.request`
         """
 
-        if url_or_manifest.startswith("<?xml"):
-            mpd = MPD(parse_xml(url_or_manifest, ignore_ns=True))
-        else:
-            res = session.http.get(url_or_manifest, **session.http.valid_request_args(**args))
-            url = res.url
+        manifest, mpd_params = cls.fetch_manifest(session, url_or_manifest, **kwargs)
 
-            urlp = list(urlparse(url))
-            urlp[2], _ = urlp[2].rsplit("/", 1)
+        try:
+            mpd = cls.parse_mpd(manifest, mpd_params)
+        except Exception as err:
+            raise PluginError(f"Failed to parse MPD manifest: {err}") from err
 
-            mpd = MPD(session.http.xml(res, ignore_ns=True), base_url=urlunparse(urlp), url=url)
-
-        video, audio = [], []
+        source = mpd_params.get("url", "MPD manifest")
+        video: List[Optional[Representation]] = [None] if with_audio_only else []
+        audio: List[Optional[Representation]] = [None] if with_video_only else []
 
         # Search for suitable video and audio representations
-        for aset in mpd.periods[0].adaptationSets:
-            if aset.contentProtection:
-                raise PluginError(f"{url} is protected by DRM")
+        for aset in mpd.periods[period].adaptationSets:
+            if aset.contentProtections:
+                raise PluginError(f"{source} is protected by DRM")
             for rep in aset.representations:
-                if rep.contentProtection:
-                    raise PluginError(f"{url} is protected by DRM")
+                if rep.contentProtections:
+                    raise PluginError(f"{source} is protected by DRM")
                 if rep.mimeType.startswith("video"):
                     video.append(rep)
-                elif rep.mimeType.startswith("audio"):
+                elif rep.mimeType.startswith("audio"):  # pragma: no branch
                     audio.append(rep)
 
         if not video:
-            video = [None]
-
+            video.append(None)
         if not audio:
-            audio = [None]
+            audio.append(None)
 
         locale = session.localization
         locale_lang = locale.language
@@ -285,34 +316,34 @@ class DASHStream(Stream):
         for aud in audio:
             if aud and aud.lang:
                 available_languages.add(aud.lang)
-                try:
+                with suppress(LookupError):
                     if locale.explicit and aud.lang and Language.get(aud.lang) == locale_lang:
                         lang = aud.lang
-                except LookupError:
-                    continue
 
         if not lang:
             # filter by the first language that appears
-            lang = audio[0] and audio[0].lang
+            lang = audio[0].lang if audio[0] else None
 
-        log.debug("Available languages for DASH audio streams: {0} (using: {1})".format(
-            ", ".join(available_languages) or "NONE",
-            lang or "n/a",
-        ))
+        log.debug(
+            f"Available languages for DASH audio streams: {', '.join(available_languages) or 'NONE'} (using: {lang or 'n/a'})",
+        )
 
         # if the language is given by the stream, filter out other languages that do not match
         if len(available_languages) > 1:
-            audio = list(filter(lambda a: a.lang is None or a.lang == lang, audio))
+            audio = [a for a in audio if a and (a.lang is None or a.lang == lang)]
 
         ret = []
         for vid, aud in itertools.product(video, audio):
-            stream = DASHStream(session, mpd, vid, aud, **args)
+            if not vid and not aud:
+                continue
+
+            stream = DASHStream(session, mpd, vid, aud, **kwargs)
             stream_name = []
 
             if vid:
-                stream_name.append("{:0.0f}{}".format(vid.height or vid.bandwidth_rounded, "p" if vid.height else "k"))
-            if audio and len(audio) > 1:
-                stream_name.append("a{:0.0f}k".format(aud.bandwidth))
+                stream_name.append(f"{vid.height or vid.bandwidth_rounded:0.0f}{'p' if vid.height else 'k'}")
+            if aud and len(audio) > 1:
+                stream_name.append(f"a{aud.bandwidth:0.0f}k")
             ret.append(("+".join(stream_name), stream))
 
         # rename duplicate streams
@@ -320,7 +351,7 @@ class DASHStream(Stream):
         for k, v in ret:
             dict_value_list[k].append(v)
 
-        def sortby_bandwidth(dash_stream: DASHStream) -> int:
+        def sortby_bandwidth(dash_stream: DASHStream) -> float:
             if dash_stream.video_representation:
                 return dash_stream.video_representation.bandwidth
             if dash_stream.audio_representation:
@@ -331,10 +362,8 @@ class DASHStream(Stream):
         for q in dict_value_list:
             items = dict_value_list[q]
 
-            try:
+            with suppress(AttributeError):
                 items = sorted(items, key=sortby_bandwidth, reverse=True)
-            except AttributeError:
-                pass
 
             for n in range(len(items)):
                 if n == 0:
@@ -343,20 +372,28 @@ class DASHStream(Stream):
                     ret_new[f"{q}_alt"] = items[n]
                 else:
                     ret_new[f"{q}_alt{n}"] = items[n]
+
         return ret_new
 
     def open(self):
-        if self.video_representation:
-            video = DASHStreamReader(self, self.video_representation.id, self.video_representation.mimeType)
+        video, audio = None, None
+        rep_video, rep_audio = self.video_representation, self.audio_representation
+
+        timestamp = now()
+
+        if rep_video:
+            video = DASHStreamReader(self, rep_video, timestamp)
+            log.debug(f"Opening DASH reader for: {rep_video.ident!r} - {rep_video.mimeType}")
             video.open()
 
-        if self.audio_representation:
-            audio = DASHStreamReader(self, self.audio_representation.id, self.audio_representation.mimeType)
+        if rep_audio:
+            audio = DASHStreamReader(self, rep_audio, timestamp)
+            log.debug(f"Opening DASH reader for: {rep_audio.ident!r} - {rep_audio.mimeType}")
             audio.open()
 
-        if self.video_representation and self.audio_representation:
+        if video and audio:
             return FFMPEGMuxer(self.session, video, audio, copyts=True).open()
-        elif self.video_representation:
+        elif video:
             return video
-        elif self.audio_representation:
+        elif audio:
             return audio
